/** @file Membrane12Rtsc.h
 * @brief Contains the interface of the Membrane12Rtsc class
 *
 * @attention All rights reserved; do not distribute without permission.
 * @author (C) Copyright 2011-2012 Paolo Greppi Universita di Genova; (C) Copyright 2012-2024 Paolo Greppi simevo s.r.l.
 */

#ifndef MEMBRANE12RTSC_H
#define MEMBRANE12RTSC_H

/* SYSTEM INCLUDES */
//

/* PROJECT INCLUDES */
#include <libpf/units/Separator.h>

/* LOCAL INCLUDES */

/* FORWARD REFERENCES */

/// Free flow, concentrated parameter RTSC membrane model
/// It is the responsability of the containing FlowSheet object to assign stageCut to stageCutEst
class Membrane12Rtsc : public Separator {
public:
  // LIFECYCLE  
  Membrane12Rtsc(Libpf::Persistency::Defaults defaults, uint32_t id=0, Persistency *persistency=nullptr, Persistent *parent=nullptr, Persistent *root=nullptr);
  
  // CUSTOM FLOWSHEET VARIABLES
  // inputs
  std::vector<Quantity> permeance; ///< permeance for each component
  Quantity Ppermeate; ///< pressure of permeate compartment
  Quantity deltaPretentate; ///< retentate pressure drop
  Quantity A; ///< membrane area

  // computed
  Quantity stageCutEst; ///< stage cut estimate
  Quantity Pretentate; ///< pressure of retentate compartment
  Quantity stageCut; ///< fraction of feed to permeate
  Quantity delta; ///< pressure ratio
  std::vector<Quantity> permeationFactor; ///< permeation factor for each component

  // OPERATIONS
  // mandatory
  const std::string &type(void) const { return type_; }
  // non-mandatory
  void go(SolutionMode solutionMode, int level);
  
  // override Separator's overrides of ObjectiveNleAdVertex virtual functions
  int calculateResidualsLocal(void);
  void setup_of(void);
private:
  static std::string type_;
  void pre_calculate(void);
  void post_calculate(void);
};  // class Membrane12Rtsc

#endif // MEMBRANE12RTSC_H
