/** @file Membrane22Linear.h
 * @brief Contains the interface of the Membrane22Linear class
 *
 * @attention All rights reserved; do not distribute without permission.
 * @author (C) Copyright 2011-2012 Paolo Greppi Universita di Genova; (C) Copyright 2012-2024 Paolo Greppi simevo s.r.l. 
 */

#ifndef MEMBRANE22LINEAR_H
#define MEMBRANE22LINEAR_H

/* SYSTEM INCLUDES */
//

/* PROJECT INCLUDES */
#include <libpf/units/MultiExchanger.h>

/* LOCAL INCLUDES */

/* FORWARD REFERENCES */
class MultiReactionTransfer;

/// Two-feeds, two-products, linear average, concentrated parameter membrane model
/// It is the responsability of the containing FlowSheet object to assign stageCut to stageCutEst
class Membrane22Linear : public MultiExchanger {
public:
  // LIFECYCLE  
  Membrane22Linear(Libpf::Persistency::Defaults defaults, uint32_t id=0, Persistency *persistency=nullptr, Persistent *parent=nullptr, Persistent *root=nullptr);
  
  // CUSTOM FLOWSHEET VARIABLES
  // inputs
  std::vector<Quantity> permeance; ///< permeance for each component
  Quantity deltaPretentate; ///< retentate compartment pressure drop
  Quantity deltaPpermeate; ///< permeate compartment pressure drop  

  // computed
  Quantity Tavg; ///< Mol-weighted average inlet temperature
  Quantity stageCutEst; ///< stage cut estimate  
  Quantity stageCut; ///< fraction of feed to permeate
  Quantity delta; ///< pressure ratio
  Quantity N; ///< feeds total mole flow
  std::vector<Quantity> permeationFactor; ///< permeation factor for each component (Ri)
  std::vector<Quantity> lambda; ///< average feed molar fractions

  // OPERATIONS
  // mandatory
  const std::string &type(void) const { return type_; }
  // non-mandatory
  void go(SolutionMode solutionMode, int level);
  
  // override MultiExchanger's overrides of ObjectiveNleAdVertex virtual functions
  int calculateResidualsLocal(void);
  void setup_of(void);  
private: 
  static std::string type_;
  void pre_calculate(void);
  void post_calculate(void);
  MultiReactionTransfer *mrt_;  
};  // class Membrane22Linear

#endif // MEMBRANE22LINEAR_H
