/** @file MembraneMulti12.h
 * @brief Contains the interface of the MembraneMulti12 class
 *
 * @attention All rights reserved; do not distribute without permission.
 * @author (C) Copyright 2011-2012 Paolo Greppi Universita di Genova; (C) Copyright 2012-2024 Paolo Greppi simevo s.r.l.
 */

#ifndef MEMBRANEMULTI12_H
#define MEMBRANEMULTI12_H

/* SYSTEM INCLUDES */
//

/* PROJECT INCLUDES */
#include <libpf/flowsheet/FlowSheet.h>

/* LOCAL INCLUDES */

/* FORWARD REFERENCES */
//

/// Free flow, multi-stage, CSTR or RTSC membrane model
/// Options:
///  - stages total number of stages, default value is 10
///  - typeV the type of the membrane model, default value is Membrane12Rtsc
class MembraneMulti12 : virtual public Model, public FlowSheet {
private:
  static std::string type_;
public:
  // LIFECYCLE  
  MembraneMulti12(Libpf::Persistency::Defaults defaults, uint32_t id=0, Persistency *persistency=nullptr, Persistent *parent=nullptr, Persistent *root=nullptr);
  
  // CUSTOM FLOWSHEET VARIABLES
  String typeV; ///< the type of the membrane model
  
  // inputs
  Quantity A; ///< membrane area
  Quantity deltaPretentate; ///< retentate pressure drop
  std::vector<Quantity> permeance; ///< permeance for each component
  Quantity Ppermeate; ///< pressure of permeate compartment

  // computed
  Quantity stageCut; ///< fraction of feed to permeate
  std::vector<Quantity> outSplit;  ///< component split fractions to permeate

  Integer stages; ///< total number of stages
  
  // OPERATIONS
  // mandatory
  void setup(void);
  const std::string &type(void) const { return type_; }
  void makeUserEquations(std::list<Assignment *>::iterator &p);
  void pre(Calculatable::SolutionMode solutionMode, int level);
  void post(Calculatable::SolutionMode solutionMode, int level);
  // non-mandatory
  // bool supportsSimultaneous(void) { return false; }
  // int maximumIterations(void) { return 1; }
private:
  std::string stagename(int i);  
};  // class MembraneMulti12

#endif // MEMBRANEMULTI12_H
